@@ -52,9 +52,12 @@ import org.apache.sysds.runtime.instructions.cp.ScalarObjectFactory;
 import org.apache.sysds.runtime.meta.DataCharacteristics;
 import org.apache.sysds.runtime.meta.MatrixCharacteristics;
 import org.apache.sysds.runtime.meta.MetaDataFormat;
+import org.apache.sysds.utils.Explain;
 
 import java.util.ArrayList;
+import java.util.Arrays;
 import java.util.HashSet;
+import java.util.List;
 import java.util.Set;
 
 /**
@@ -72,8 +75,7 @@ import java.util.Set;
  *  recompile once functions, the removal of unnecessary checkpoints, and the 
  *  global removal of constant binary operations such as X * ones.
  */
-public class InterProceduralAnalysis 
-{
+public class InterProceduralAnalysis {
 	private static final Log LOG = LogFactory.getLog(InterProceduralAnalysis.class.getName());
 
 	//internal configuration parameters
@@ -101,7 +103,7 @@ public class InterProceduralAnalysis
 	
 	//set IPA passes to apply in order 
 	private final ArrayList<IPAPass> _passes;
-	
+
 	/**
 	 * Creates a handle for performing inter-procedural analysis
 	 * for a given DML program and its associated HOP DAGs. This
@@ -112,12 +114,16 @@ public class InterProceduralAnalysis
 	 * @param dmlp The DML program to analyze
 	 */
 	public InterProceduralAnalysis(DMLProgram dmlp) {
-		//analyzes the function call graph 
+		//analyzes the function call graph
 		_prog = dmlp;
 		_sb = null;
 		_fgraph = new FunctionCallGraph(dmlp);
+		if( LOG.isDebugEnabled() ) {
+			LOG.debug("IPA: Initial FunctionCallGraph: \n--MAIN PROGRAM\n" + 
+				Explain.explainFunctionCallGraph(_fgraph, new HashSet<String>(), null, 1));
+		}
 		
-		//create order list of IPA passes
+		//create ordered list of IPA passes
 		_passes = new ArrayList<>();
 		_passes.add(new IPAPassRemoveUnusedFunctions());
 		_passes.add(new IPAPassFlagFunctionsRecompileOnce());
@@ -125,6 +131,7 @@ public class InterProceduralAnalysis
 		_passes.add(new IPAPassRemoveConstantBinaryOps());
 		_passes.add(new IPAPassPropagateReplaceLiterals());
 		_passes.add(new IPAPassInlineFunctions());
+		_passes.add(new IPAPassReplaceEvalFunctionCalls());
 		_passes.add(new IPAPassEliminateDeadCode());
 		_passes.add(new IPAPassFlagNonDeterminism());
 		//note: apply rewrites last because statement block rewrites
@@ -157,7 +164,6 @@ public class InterProceduralAnalysis
 	 * 
 	 * @param repetitions number of IPA rounds 
 	 */
-	@SuppressWarnings("null")
 	public void analyzeProgram(int repetitions) {
 		//sanity check for valid number of repetitions
 		if( repetitions <= 0 )
@@ -213,17 +219,23 @@ public class InterProceduralAnalysis
 						+ " repetitions due to reached fixpoint.");
 				break;
 			}
+			lastSizes = fcallSizes;
 			
 			//step 3: rebuild function call graph if necessary
 			if( rebuildFGraph && i < repetitions-1 )
 				_fgraph = new FunctionCallGraph(_prog);
 		}
 		
-		//cleanup pass: remove unused functions
+		//cleanup passes: remove unused functions, CLA workload extraction
 		FunctionCallGraph graph2 = new FunctionCallGraph(_prog);
-		IPAPass rmFuns = new IPAPassRemoveUnusedFunctions();
-		if( rmFuns.isApplicable(graph2) )
-			rmFuns.rewriteProgram(_prog, graph2, null);
+		List<IPAPass> fpasses = Arrays.asList(
+			new IPAPassRemoveUnusedFunctions(),
+			new IPAPassCompressionWorkloadAnalysis(), // workload-aware compression
+			new IPAPassApplyStaticAndDynamicHopRewrites(),  //split after compress
+			new IPAPassRewriteFederatedPlan());
+		for(IPAPass pass : fpasses)
+			if( pass.isApplicable(graph2) )
+				pass.rewriteProgram(_prog, graph2, null);
 	}
 	
 	public Set<String> analyzeSubProgram() {
@@ -521,10 +533,13 @@ public class InterProceduralAnalysis
 		ArrayList<Hop> inputOps = fop.getInput();
 		String fkey = fop.getFunctionKey();
 		
-		for( int i=0; i<funArgNames.length; i++ )
-		{
+		//iterate over all parameters (with robustness for missing parameters)
+		for( int i=0; i<Math.min(inputOps.size(), funArgNames.length); i++ ) {
 			//create mapping between input hops and vars
 			DataIdentifier dat = fstmt.getInputParam(funArgNames[i]);
+			if( dat == null )
+				throw new HopsException("Failed IPA: function argument '"+funArgNames[i]+"' "
+					+ "does not exist in function signature of "+fop.getFunctionKey()+".");
 			Hop input = inputOps.get(i);
 			
 			if( input.getDataType()==DataType.MATRIX )
