@@ -19,7 +19,7 @@
 
 package org.apache.sysds.hops.rewrite;
 
-import org.apache.commons.lang.ArrayUtils;
+import org.apache.commons.lang3.ArrayUtils;
 import org.apache.commons.logging.Log;
 import org.apache.commons.logging.LogFactory;
 import org.apache.sysds.api.DMLScript;
@@ -569,8 +569,7 @@ public class HopRewriteUtils {
 	
 	public static DataOp createTransientRead(String name, MatrixObject mo) {
 		DataOp tread = new DataOp(name, DataType.MATRIX, ValueType.FP64, OpOpData.TRANSIENTREAD,
-			null, mo.getNumRows(), mo.getNumColumns(), mo.getNnz(), UpdateType.COPY,
-			(int)mo.getBlocksize());
+			null, mo.getNumRows(), mo.getNumColumns(), mo.getNnz(), UpdateType.COPY, mo.getBlocksize());
 		tread.setVisited();
 		copyLineNumbers(mo, tread);
 		tread.setFileName(name);
@@ -1133,6 +1132,15 @@ public class HopRewriteUtils {
 		return hop instanceof DataOp && ((DataOp)hop).getOp()==type;
 	}
 	
+	public static boolean isData(Hop hop, OpOpData type, DataType dt) {
+		return isData(hop, type) && hop.getDataType()==dt;
+	}
+	
+	public static boolean isTransformEncode(Hop hop){
+		return hop instanceof FunctionOp 
+			&& (((FunctionOp)hop).getFunctionName().equalsIgnoreCase("transformencode"));
+	}
+
 	public static boolean isBinaryMatrixColVectorOperation(Hop hop) {
 		return hop instanceof BinaryOp 
 			&& hop.getInput().get(0).getDataType().isMatrix() && hop.getInput().get(1).getDataType().isMatrix()
@@ -1317,10 +1325,11 @@ public class HopRewriteUtils {
 		//starting row and column ranges of 1 in order to guard against
 		//invalid modifications in the presence of invalid index ranges
 		//(e.g., X[,2] on a column vector needs to throw an error)
-		return isEqualSize(hop, hop.getInput().get(0))
-			&& !(hop.getDim1()==1 && hop.getDim2()==1)
-			&& isLiteralOfValue(hop.getInput().get(1), 1)  //rl
-			&& isLiteralOfValue(hop.getInput().get(3), 1); //cl
+		return ((IndexingOp)hop).isAllRowsAndCols()
+			|| (isEqualSize(hop, hop.getInput().get(0))
+				&& !(hop.getDim1()==1 && hop.getDim2()==1)
+				&& isLiteralOfValue(hop.getInput().get(1), 1)  //rl
+				&& isLiteralOfValue(hop.getInput().get(3), 1)); //cl
 	}
 	
 	public static boolean isScalarMatrixBinaryMult( Hop hop ) {
@@ -1549,6 +1558,29 @@ public class HopRewriteUtils {
 		return sb instanceof WhileStatementBlock
 			|| sb instanceof ForStatementBlock; //incl parfor
 	}
+
+	// Return true if this loop contains only basic blocks
+	public static boolean isLastLevelLoopStatementBlock (StatementBlock sb) {
+		if (!isLoopStatementBlock(sb))
+			return false;
+		if (sb instanceof WhileStatementBlock) {
+			WhileStatement wstmt = (WhileStatement) sb.getStatement(0);
+			if (wstmt.getBody().isEmpty())
+				return false;
+			for(StatementBlock csb : wstmt.getBody())
+				if (!isLastLevelStatementBlock(csb))
+					return false;
+		}
+		else if (sb instanceof ForStatementBlock) {
+			ForStatement fstmt = (ForStatement) sb.getStatement(0);
+			if (fstmt.getBody().isEmpty())
+				return false;
+			for(StatementBlock csb : fstmt.getBody())
+				if(!isLastLevelStatementBlock(csb))
+					return false;
+		}
+		return true;
+	}
 	
 	public static long getMaxNrowInput(Hop hop) {
 		return getMaxInputDim(hop, true);
