@@ -49,6 +49,7 @@ public class ParameterizedBuiltinFunctionExpression extends DataIdentifier
 	public static final String TF_FN_PARAM_DATA = "target";
 	public static final String TF_FN_PARAM_MTD2 = "meta";
 	public static final String TF_FN_PARAM_SPEC = "spec";
+	public static final String TF_FN_PARAM_EMBD = "embedding";
 	public static final String LINEAGE_TRACE = "lineage";
 	public static final String TF_FN_PARAM_MTD = "transformPath"; //NOTE MB: for backwards compatibility
 	
@@ -202,6 +203,10 @@ public class ParameterizedBuiltinFunctionExpression extends DataIdentifier
 			validateReplace(output, conditional);
 			break;
 		
+		case CONTAINS:
+			validateContains(output, conditional);
+			break;
+		
 		case ORDER:
 			validateOrder(output, conditional);
 			break;
@@ -246,8 +251,21 @@ public class ParameterizedBuiltinFunctionExpression extends DataIdentifier
 			validateParamserv(output, conditional);
 			break;
 
+		case COUNT_DISTINCT:
+			validateCountDistinct(output, conditional);
+			break;
+
 		case COUNT_DISTINCT_APPROX:
-			validateCountDistinctApprox(output, conditional);
+			validateCountDistinctApprox(output, conditional, false);
+			break;
+
+		case COUNT_DISTINCT_APPROX_ROW:
+		case COUNT_DISTINCT_APPROX_COL:
+			validateCountDistinctApprox(output, conditional, true);
+			break;
+
+		case UNIQUE:
+			validateUnique(output, conditional);
 			break;
 
 		default: //always unconditional (because unsupported operation)
@@ -321,7 +339,7 @@ public class ParameterizedBuiltinFunctionExpression extends DataIdentifier
 			Statement.PS_VAL_FUN, Statement.PS_MODE, Statement.PS_UPDATE_TYPE, Statement.PS_FREQUENCY, Statement.PS_EPOCHS,
 			Statement.PS_BATCH_SIZE, Statement.PS_PARALLELISM, Statement.PS_SCHEME, Statement.PS_FED_RUNTIME_BALANCING,
 			Statement.PS_FED_WEIGHTING, Statement.PS_HYPER_PARAMS, Statement.PS_CHECKPOINTING, Statement.PS_SEED, Statement.PS_NBATCHES,
-			Statement.PS_MODELAVG, Statement.PS_HE);
+			Statement.PS_MODELAVG, Statement.PS_HE, Statement.PS_NUM_BACKUP_WORKERS);
 		checkInvalidParameters(getOpCode(), getVarParams(), valid);
 
 		// check existence and correctness of parameters
@@ -353,7 +371,46 @@ public class ParameterizedBuiltinFunctionExpression extends DataIdentifier
 		output.setBlocksize(-1);
 	}
 
-	private void validateCountDistinctApprox(DataIdentifier output, boolean conditional) {
+	private void validateCountDistinct(DataIdentifier output, boolean conditional) {
+		HashMap<String, Expression> varParams = getVarParams();
+
+		// "data" is the only parameter that is allowed to be unnamed
+		if (varParams.containsKey(null)) {
+			varParams.put("data", varParams.remove(null));
+		}
+
+		// Validate the number of parameters
+		String fname = getOpCode().getName();
+		String usageMessage = "function " + fname + " takes at least 1 and at most 2 parameters";
+		if (varParams.size() < 1) {
+			raiseValidateError("Too few parameters: " + usageMessage, conditional);
+		}
+
+		if (varParams.size() > 2) {
+			raiseValidateError("Too many parameters: " + usageMessage, conditional);
+		}
+
+		// Check parameter names are valid
+		Set<String> validParameterNames = CollectionUtils.asSet("data", "dir");
+		checkInvalidParameters(getOpCode(), varParams, validParameterNames);
+
+		// Check parameter expression data types match expected
+		checkDataType(false, fname, "data", DataType.MATRIX, conditional);
+		checkDataValueType(false, fname, "data", DataType.MATRIX, ValueType.FP64, conditional);
+
+		// We need the dimensions of the input matrix to determine the output matrix characteristics
+		// Validate data parameter, lookup previously defined var or resolve expression
+		Identifier dataId = varParams.get("data").getOutput();
+		if (dataId == null) {
+			raiseValidateError("Cannot parse input parameter \"data\" to function " + fname, conditional);
+		}
+
+		checkStringParam(true, fname, "dir", conditional);
+		// Check data value of "dir" parameter
+		validateCountDistinctAggregationDirection(dataId, output);
+	}
+
+	private void validateCountDistinctApprox(DataIdentifier output, boolean conditional, boolean isDirectionAlias) {
 		Set<String> validTypeNames = CollectionUtils.asSet("KMV");
 		HashMap<String, Expression> varParams = getVarParams();
 
@@ -364,13 +421,26 @@ public class ParameterizedBuiltinFunctionExpression extends DataIdentifier
 
 		// Validate the number of parameters
 		String fname = getOpCode().getName();
-		String usageMessage = "function " + fname + " takes at least 1 and at most 3 parameters";
-		if (varParams.size() < 1) {
-			raiseValidateError("Too few parameters: " + usageMessage, conditional);
-		}
+		if (!isDirectionAlias) {
+			// Function is not an alias, so we have to check for all 3 permissible parameters
+			String usageMessage = "function " + fname + " takes at least 1 and at most 3 parameters";
+			if (varParams.size() < 1) {
+				raiseValidateError("Too few parameters: " + usageMessage, conditional);
+			}
 
-		if (varParams.size() > 3) {
-			raiseValidateError("Too many parameters: " + usageMessage, conditional);
+			if (varParams.size() > 3) {
+				raiseValidateError("Too many parameters: " + usageMessage, conditional);
+			}
+		} else {
+			// The direction is fixed for function aliases
+			String usageMessage = "function " + fname + " takes at least 1 and at most 2 parameters";
+			if (varParams.size() < 1) {
+				raiseValidateError("Too few parameters: " + usageMessage, conditional);
+			}
+
+			if (varParams.size() > 2) {
+				raiseValidateError("Too many parameters: " + usageMessage, conditional);
+			}
 		}
 
 		// Check parameter names are valid
@@ -390,7 +460,7 @@ public class ParameterizedBuiltinFunctionExpression extends DataIdentifier
 
 		checkStringParam(true, fname, "type", conditional);
 		// Check data value of "type" parameter
-		if (varParams.keySet().contains("type")) {
+		if (varParams.containsKey("type")) {
 			String typeString = varParams.get("type").toString().toUpperCase();
 			if (!validTypeNames.contains(typeString)) {
 				raiseValidateError("Unrecognized type for optional parameter " + typeString, conditional);
@@ -400,15 +470,22 @@ public class ParameterizedBuiltinFunctionExpression extends DataIdentifier
 			addVarParam("type", new StringIdentifier("KMV", this));
 		}
 
-		checkStringParam(true, fname, "dir", conditional);
-		// Check data value of "dir" parameter
-		if (varParams.keySet().contains("dir")) {
-			String directionString = varParams.get("dir").toString().toUpperCase();
+		if (!isDirectionAlias) {
+			checkStringParam(true, fname, "dir", conditional);
+			// Check data value of "dir" parameter
+			validateCountDistinctAggregationDirection(dataId, output);
+		}
+	}
+
+	private void validateCountDistinctAggregationDirection(Identifier dataId, DataIdentifier output) {
+		HashMap<String, Expression> varParams = getVarParams();
+		if (varParams.containsKey("dir")) {
+			String inputDirectionString = varParams.get("dir").toString().toUpperCase();
 
 			// Set output type and dimensions based on direction
 
 			// "r" -> count across all rows, resulting in a Mx1 matrix
-			if (directionString.equals(Types.Direction.Row.toString())) {
+			if (inputDirectionString.equals(Types.Direction.Row.toString())) {
 				output.setDataType(DataType.MATRIX);
 				output.setDimensions(dataId.getDim1(), 1);
 				output.setBlocksize(dataId.getBlocksize());
@@ -416,7 +493,7 @@ public class ParameterizedBuiltinFunctionExpression extends DataIdentifier
 				output.setNnz(dataId.getDim1());
 
 			// "c" -> count across all cols, resulting in a 1xN matrix
-			} else if (directionString.equals(Types.Direction.Col.toString())) {
+			} else if (inputDirectionString.equals(Types.Direction.Col.toString())) {
 				output.setDataType(DataType.MATRIX);
 				output.setDimensions(1, dataId.getDim2());
 				output.setBlocksize(dataId.getBlocksize());
@@ -424,20 +501,18 @@ public class ParameterizedBuiltinFunctionExpression extends DataIdentifier
 				output.setNnz(dataId.getDim2());
 
 			// "rc" -> count across all rows and cols in input matrix, resulting in a single value
-			} else if (directionString.equals(Types.Direction.RowCol.toString())) {
+			} else if (inputDirectionString.equals(Types.Direction.RowCol.toString())) {
 				output.setDataType(DataType.SCALAR);
 				output.setDimensions(0, 0);
 				output.setBlocksize(0);
 				output.setValueType(ValueType.INT64);
 				output.setNnz(1);
 
-			// unrecognized value for "dir" parameter, should "cr" be valid?
+			// unrecognized value for "dir" parameter
 			} else {
-				raiseValidateError("Invalid argument: " + directionString + " is not recognized");
+				raiseValidateError("Invalid argument: " + inputDirectionString + " is not recognized");
 			}
-
-		// default to dir="rc"
-		} else {
+		} else {  // default to dir="rc"
 			output.setDataType(DataType.SCALAR);
 			output.setDimensions(0, 0);
 			output.setBlocksize(0);
@@ -446,6 +521,67 @@ public class ParameterizedBuiltinFunctionExpression extends DataIdentifier
 		}
 	}
 
+	private void validateUnique(DataIdentifier output, boolean conditional) {
+		HashMap<String, Expression> varParams = getVarParams();
+
+		// "data" is the only parameter that is allowed to be unnamed
+		if (varParams.containsKey(null)) {
+			varParams.put("data", varParams.remove(null));
+		}
+
+		// Validate the number of parameters
+		String fname = getOpCode().getName();
+		String usageMessage = "function " + fname + " takes at least 1 and at most 2 parameters";
+		if (varParams.size() < 1) {
+			raiseValidateError("Too few parameters: " + usageMessage, conditional);
+		}
+
+		if (varParams.size() > 2) {
+			raiseValidateError("Too many parameters: " + usageMessage, conditional);
+		}
+
+		// Check parameter names are valid
+		Set<String> validParameterNames = CollectionUtils.asSet("data", "dir");
+		checkInvalidParameters(getOpCode(), varParams, validParameterNames);
+
+		// Check parameter expression data types match expected
+		checkDataType(false, fname, "data", DataType.MATRIX, conditional);
+		checkDataValueType(false, fname, "data", DataType.MATRIX, ValueType.FP64, conditional);
+
+		// We need the dimensions of the input matrix to determine the output matrix characteristics
+		// Validate data parameter, lookup previously defined var or resolve expression
+		Identifier dataId = varParams.get("data").getOutput();
+		if (dataId == null) {
+			raiseValidateError("Cannot parse input parameter \"data\" to function " + fname, conditional);
+		}
+
+		checkStringParam(true, fname, "dir", conditional);
+		// Check data value of "dir" parameter
+		validateUniqueAggregationDirection(dataId, output);
+	}
+
+	private void validateUniqueAggregationDirection(Identifier dataId, DataIdentifier output) {
+		HashMap<String, Expression> varParams = getVarParams();
+		if (varParams.containsKey("dir")) {
+			String inputDirectionString = varParams.get("dir").toString().toUpperCase();
+
+			// unrecognized value for "dir" parameter
+			if (!inputDirectionString.equals(Types.Direction.Row.toString())
+					&& !inputDirectionString.equals(Types.Direction.Col.toString())
+					&& !inputDirectionString.equals(Types.Direction.RowCol.toString())) {
+				raiseValidateError("Invalid argument: " + inputDirectionString + " is not recognized");
+			}
+		}
+
+		// rc/r/c -> unique return value is the same as the input in the worst case
+		// default to dir="rc"
+		output.setDataType(DataType.MATRIX);
+		output.setDimensions(dataId.getDim1(), dataId.getDim2());
+		output.setBlocksize(dataId.getBlocksize());
+		output.setValueType(ValueType.FP64);
+		output.setNnz(dataId.getNnz());
+	}
+
 	private void checkStringParam(boolean optional, String fname, String pname, boolean conditional) {
 		Expression param = getVarParam(pname);
 		if (param == null) {
@@ -482,11 +618,14 @@ public class ParameterizedBuiltinFunctionExpression extends DataIdentifier
 		//validate data / metadata (recode maps)
 		checkDataType(false, "transformapply", TF_FN_PARAM_DATA, DataType.FRAME, conditional);
 		checkDataType(false, "transformapply", TF_FN_PARAM_MTD2, DataType.FRAME, conditional);
-		
+
 		//validate specification
 		checkDataValueType(false, "transformapply", TF_FN_PARAM_SPEC, DataType.SCALAR, ValueType.STRING, conditional);
 		validateTransformSpec(TF_FN_PARAM_SPEC, conditional);
-		
+
+		//validate additional argument for word_embeddings tranform
+		checkDataType(true, "transformapply", TF_FN_PARAM_EMBD, DataType.MATRIX, conditional);
+
 		//set output dimensions
 		output.setDataType(DataType.MATRIX);
 		output.setValueType(ValueType.FP64);
@@ -594,28 +733,31 @@ public class ParameterizedBuiltinFunctionExpression extends DataIdentifier
 		output.setDimensions(in.getDim1(), in.getDim2());
 	}
 	
+	private void validateContains(DataIdentifier output, boolean conditional) {
+		//check existence and correctness of arguments
+		Expression target = getVarParam("target");
+		checkTargetParam(target, conditional);
+		
+		Expression pattern = getVarParam("pattern");
+		if(pattern == null)
+			raiseValidateError("Named parameter 'pattern' missing. Please specify the input matrix.",
+				conditional, LanguageErrorCodes.INVALID_PARAMETERS);
+		if(!(pattern.getOutput().getDataType().isScalar()
+			||pattern.getOutput().getDataType().isMatrix()) )
+			raiseValidateError("Named parameter 'pattern' must be a scalar or matrix.",
+				conditional, LanguageErrorCodes.INVALID_PARAMETERS);
+		//set boolean scalar 
+		output.setBooleanProperties();
+	}
+	
 	private void validateReplace(DataIdentifier output, boolean conditional) {
 		//check existence and correctness of arguments
 		Expression target = getVarParam("target");
 		if( target.getOutput().getDataType() != DataType.FRAME ){
 			checkTargetParam(target, conditional);
 		}
-		
-		Expression pattern = getVarParam("pattern");
-		if( pattern==null ) {
-			raiseValidateError("Named parameter 'pattern' missing. Please specify the replacement pattern.", conditional, LanguageErrorCodes.INVALID_PARAMETERS);
-		}
-		else if( pattern.getOutput().getDataType() != DataType.SCALAR ){				
-			raiseValidateError("Replacement pattern 'pattern' is of type '"+pattern.getOutput().getDataType()+"'. Please, specify a scalar replacement pattern.", conditional, LanguageErrorCodes.INVALID_PARAMETERS);
-		}	
-		
-		Expression replacement = getVarParam("replacement");
-		if( replacement==null ) {
-			raiseValidateError("Named parameter 'replacement' missing. Please specify the replacement value.", conditional, LanguageErrorCodes.INVALID_PARAMETERS);
-		}
-		else if( replacement.getOutput().getDataType() != DataType.SCALAR ){	
-			raiseValidateError("Replacement value 'replacement' is of type '"+replacement.getOutput().getDataType()+"'. Please, specify a scalar replacement value.", conditional, LanguageErrorCodes.INVALID_PARAMETERS);
-		}	
+		checkScalarParam("replace", "pattern", conditional);
+		checkScalarParam("replace", "replacement", conditional);
 		
 		// Output is a matrix with same dims as input
 		output.setDataType(target.getOutput().getDataType());
@@ -625,6 +767,19 @@ public class ParameterizedBuiltinFunctionExpression extends DataIdentifier
 			output.setValueType(ValueType.FP64);
 		output.setDimensions(target.getOutput().getDim1(), target.getOutput().getDim2());
 	}
+	
+	private void checkScalarParam(String group, String param, boolean conditional) {
+		Expression eparam = getVarParam(param);
+		if( eparam==null ) {
+			raiseValidateError("Named parameter '"+param+"' missing. Please specify the "+group+" pattern.",
+				conditional, LanguageErrorCodes.INVALID_PARAMETERS);
+		}
+		else if( eparam.getOutput().getDataType() != DataType.SCALAR ){
+			raiseValidateError(group + " parameter '"+param+"' is of type '"
+				+ eparam.getOutput().getDataType()+"'. Please, specify a scalar "+param+".",
+				conditional, LanguageErrorCodes.INVALID_PARAMETERS);
+		}
+	}
 
 	private void validateOrder(DataIdentifier output, boolean conditional) {
 		//check existence and correctness of arguments
