@@ -30,7 +30,7 @@ import java.util.Iterator;
 import java.util.List;
 import java.util.concurrent.Future;
 
-import org.apache.commons.lang.NotImplementedException;
+import org.apache.commons.lang3.NotImplementedException;
 import org.apache.commons.logging.Log;
 import org.apache.commons.logging.LogFactory;
 import org.apache.commons.math3.random.Well1024a;
@@ -51,11 +51,13 @@ import org.apache.sysds.runtime.compress.lib.CLALibCompAgg;
 import org.apache.sysds.runtime.compress.lib.CLALibDecompress;
 import org.apache.sysds.runtime.compress.lib.CLALibMMChain;
 import org.apache.sysds.runtime.compress.lib.CLALibMatrixMult;
+import org.apache.sysds.runtime.compress.lib.CLALibMerge;
 import org.apache.sysds.runtime.compress.lib.CLALibRexpand;
 import org.apache.sysds.runtime.compress.lib.CLALibScalar;
 import org.apache.sysds.runtime.compress.lib.CLALibSlice;
 import org.apache.sysds.runtime.compress.lib.CLALibSquash;
 import org.apache.sysds.runtime.compress.lib.CLALibTSMM;
+import org.apache.sysds.runtime.compress.lib.CLALibTernaryOp;
 import org.apache.sysds.runtime.compress.lib.CLALibUnary;
 import org.apache.sysds.runtime.compress.lib.CLALibUtils;
 import org.apache.sysds.runtime.controlprogram.caching.MatrixObject.UpdateType;
@@ -63,9 +65,7 @@ import org.apache.sysds.runtime.controlprogram.parfor.stat.InfrastructureAnalyze
 import org.apache.sysds.runtime.data.DenseBlock;
 import org.apache.sysds.runtime.data.SparseBlock;
 import org.apache.sysds.runtime.data.SparseRow;
-import org.apache.sysds.runtime.functionobjects.MinusMultiply;
-import org.apache.sysds.runtime.functionobjects.PlusMultiply;
-import org.apache.sysds.runtime.functionobjects.TernaryValueFunction.ValueFunctionWithConstant;
+import org.apache.sysds.runtime.functionobjects.SwapIndex;
 import org.apache.sysds.runtime.instructions.InstructionUtils;
 import org.apache.sysds.runtime.instructions.cp.CM_COV_Object;
 import org.apache.sysds.runtime.instructions.cp.ScalarObject;
@@ -73,14 +73,12 @@ import org.apache.sysds.runtime.instructions.spark.data.IndexedMatrixValue;
 import org.apache.sysds.runtime.matrix.data.CTableMap;
 import org.apache.sysds.runtime.matrix.data.IJV;
 import org.apache.sysds.runtime.matrix.data.LibMatrixDatagen;
-import org.apache.sysds.runtime.matrix.data.LibMatrixTercell;
 import org.apache.sysds.runtime.matrix.data.MatrixBlock;
 import org.apache.sysds.runtime.matrix.data.MatrixIndexes;
 import org.apache.sysds.runtime.matrix.data.MatrixValue;
 import org.apache.sysds.runtime.matrix.data.RandomMatrixGenerator;
 import org.apache.sysds.runtime.matrix.operators.AggregateBinaryOperator;
 import org.apache.sysds.runtime.matrix.operators.AggregateOperator;
-import org.apache.sysds.runtime.matrix.operators.AggregateTernaryOperator;
 import org.apache.sysds.runtime.matrix.operators.AggregateUnaryOperator;
 import org.apache.sysds.runtime.matrix.operators.BinaryOperator;
 import org.apache.sysds.runtime.matrix.operators.CMOperator;
@@ -98,6 +96,11 @@ public class CompressedMatrixBlock extends MatrixBlock {
 	private static final Log LOG = LogFactory.getLog(CompressedMatrixBlock.class.getName());
 	private static final long serialVersionUID = 73193720143154058L;
 
+	/**
+	 * Debugging flag for Compressed Matrices
+	 */
+	public static boolean debug = false;
+
 	/**
 	 * Column groups
 	 */
@@ -229,6 +232,23 @@ public class CompressedMatrixBlock extends MatrixBlock {
 		return decompress(1);
 	}
 
+	/**
+	 * Get the column group allocated and associated with a specific column Id;
+	 * 
+	 * There is some search involved in this since we do not know where to look for the column and which Column group
+	 * contains the value.
+	 * 
+	 * @param id The column id or number we try to find
+	 * @return The column group for that column
+	 */
+	public AColGroup getColGroupForColumn(int id) {
+		for(AColGroup g : _colGroups) {
+			if(g.getColIndices().contains(id))
+				return g;
+		}
+		return null;
+	}
+
 	/**
 	 * Decompress block into a MatrixBlock
 	 * 
@@ -255,7 +275,7 @@ public class CompressedMatrixBlock extends MatrixBlock {
 
 	@Override
 	public void putInto(MatrixBlock target, int rowOffset, int colOffset, boolean sparseCopyShallow) {
-		CLALibDecompress.decompressTo(this, target, rowOffset, colOffset, 1);
+		CLALibDecompress.decompressTo(this, target, rowOffset, colOffset, 1, false);
 	}
 
 	/**
@@ -404,16 +424,18 @@ public class CompressedMatrixBlock extends MatrixBlock {
 		if(nonZeros > 0 && estDisk > estimateUncompressed) {
 			// If the size of this matrixBlock is smaller in uncompressed format, then
 			// decompress and save inside an uncompressed column group.
-			MatrixBlock uncompressed = getUncompressed(
-				"smaller serialization size: compressed: " + estDisk + " vs uncompressed: " + estimateUncompressed);
-			ColGroupUncompressed cg = (ColGroupUncompressed) ColGroupUncompressed.create(uncompressed);
-
-			if( estDisk / 10 > estimateUncompressed){
-				LOG.error(this);
-			}
+			final String message = "smaller serialization size: compressed: " + estDisk + " vs uncompressed: "
+				+ estimateUncompressed;
+			final MatrixBlock uncompressed = getUncompressed(message);
+			uncompressed.examSparsity(true);
+			// Here only Empty or Uncompressed should be returned.
+			AColGroup cg = ColGroupUncompressed.create(uncompressed);
 			allocateColGroup(cg);
+			// update non zeros, if not fully correct in compressed block
 			nonZeros = cg.getNumberNonZeros(rlen);
-			// clear the soft reference to the decompressed version, since the one column group is perfectly,
+
+			// Clear the soft reference to the decompressed version,
+			// since the one column group is perfectly,
 			// representing the decompressed version.
 			clearSoftReferenceToDecompressed();
 		}
@@ -489,7 +511,8 @@ public class CompressedMatrixBlock extends MatrixBlock {
 	}
 
 	@Override
-	public MatrixBlock chainMatrixMultOperations(MatrixBlock v, MatrixBlock w, MatrixBlock out, ChainType ctype, int k) {
+	public MatrixBlock chainMatrixMultOperations(MatrixBlock v, MatrixBlock w, MatrixBlock out, ChainType ctype,
+		int k) {
 
 		checkMMChain(ctype, v, w);
 		// multi-threaded MMChain of single uncompressed ColGroup
@@ -566,26 +589,21 @@ public class CompressedMatrixBlock extends MatrixBlock {
 
 	@Override
 	public MatrixBlock reorgOperations(ReorgOperator op, MatrixValue ret, int startRow, int startColumn, int length) {
-		// Allow transpose to be compressed output. In general we need to have a transposed flag on
-		// the compressed matrix. https://issues.apache.org/jira/browse/SYSTEMDS-3025
-		printDecompressWarning(op.getClass().getSimpleName() + " -- " + op.fn.getClass().getSimpleName());
-		MatrixBlock tmp = decompress(op.getNumThreads());
-		return tmp.reorgOperations(op, ret, startRow, startColumn, length);
-	}
+		if(op.fn instanceof SwapIndex && this.getNumColumns() == 1) {
+			MatrixBlock tmp = decompress(op.getNumThreads());
+			long nz = tmp.setNonZeros(tmp.getNonZeros());
+			tmp = new MatrixBlock(tmp.getNumColumns(), tmp.getNumRows(), tmp.getDenseBlockValues());
+			tmp.setNonZeros(nz);
+			return tmp;
+		}
+		else {
+			// Allow transpose to be compressed output. In general we need to have a transposed flag on
+			// the compressed matrix. https://issues.apache.org/jira/browse/SYSTEMDS-3025
+			String message = op.getClass().getSimpleName() + " -- " + op.fn.getClass().getSimpleName();
+			MatrixBlock tmp = getUncompressed(message, op.getNumThreads());
+			return tmp.reorgOperations(op, ret, startRow, startColumn, length);
+		}
 
-	@Override
-	public String toString() {
-		StringBuilder sb = new StringBuilder();
-		sb.append("CompressedMatrixBlock:");
-		sb.append("\nCols:" + getNumColumns() + " Rows:" + getNumRows() + " Overlapping: " + isOverlapping() + " nnz: "
-			+ nonZeros);
-		if(_colGroups != null)
-			for(AColGroup cg : _colGroups) {
-				sb.append("\n" + cg);
-			}
-		else
-			sb.append("\nEmptyColGroups");
-		return sb.toString();
 	}
 
 	public boolean isOverlapping() {
@@ -617,7 +635,8 @@ public class CompressedMatrixBlock extends MatrixBlock {
 
 	@Override
 	public boolean containsValue(double pattern) {
-		if(isOverlapping())
+		// Only if pattern is a finite value and overlapping then decompress.
+		if(isOverlapping() && Double.isFinite(pattern))
 			return getUncompressed("ContainsValue").containsValue(pattern);
 		else {
 			for(AColGroup g : _colGroups)
@@ -781,24 +800,6 @@ public class CompressedMatrixBlock extends MatrixBlock {
 		return getUncompressed("sortOperations").sortOperations(right, result);
 	}
 
-	@Override
-	public MatrixBlock aggregateTernaryOperations(MatrixBlock m1, MatrixBlock m2, MatrixBlock m3, MatrixBlock ret,
-		AggregateTernaryOperator op, boolean inCP) {
-		boolean m1C = m1 instanceof CompressedMatrixBlock;
-		boolean m2C = m2 instanceof CompressedMatrixBlock;
-		boolean m3C = m3 instanceof CompressedMatrixBlock;
-		printDecompressWarning("aggregateTernaryOperations " + op.aggOp.getClass().getSimpleName() + " "
-			+ op.indexFn.getClass().getSimpleName() + " " + op.aggOp.increOp.fn.getClass().getSimpleName() + " "
-			+ op.binaryFn.getClass().getSimpleName() + " m1,m2,m3 " + m1C + " " + m2C + " " + m3C);
-		MatrixBlock left = getUncompressed(m1);
-		MatrixBlock right1 = getUncompressed(m2);
-		MatrixBlock right2 = getUncompressed(m3);
-		ret = left.aggregateTernaryOperations(left, right1, right2, ret, op, inCP);
-		if(ret.getNumRows() == 0 || ret.getNumColumns() == 0)
-			throw new DMLCompressionException("Invalid output");
-		return ret;
-	}
-
 	@Override
 	public MatrixBlock uaggouterchainOperations(MatrixBlock mbLeft, MatrixBlock mbRight, MatrixBlock mbOut,
 		BinaryOperator bOp, AggregateUnaryOperator uaggOp) {
@@ -878,54 +879,7 @@ public class CompressedMatrixBlock extends MatrixBlock {
 
 	@Override
 	public MatrixBlock ternaryOperations(TernaryOperator op, MatrixBlock m2, MatrixBlock m3, MatrixBlock ret) {
-
-		// prepare inputs
-		final int r1 = getNumRows();
-		final int r2 = m2.getNumRows();
-		final int r3 = m3.getNumRows();
-		final int c1 = getNumColumns();
-		final int c2 = m2.getNumColumns();
-		final int c3 = m3.getNumColumns();
-		final boolean s1 = (r1 == 1 && c1 == 1);
-		final boolean s2 = (r2 == 1 && c2 == 1);
-		final boolean s3 = (r3 == 1 && c3 == 1);
-		final double d1 = s1 ? quickGetValue(0, 0) : Double.NaN;
-		final double d2 = s2 ? m2.quickGetValue(0, 0) : Double.NaN;
-		final double d3 = s3 ? m3.quickGetValue(0, 0) : Double.NaN;
-		final int m = Math.max(Math.max(r1, r2), r3);
-		final int n = Math.max(Math.max(c1, c2), c3);
-
-		ternaryOperationCheck(s1, s2, s3, m, r1, r2, r3, n, c1, c2, c3);
-
-		final boolean PM_Or_MM = (op.fn instanceof PlusMultiply || op.fn instanceof MinusMultiply);
-		if(PM_Or_MM && ((s2 && d2 == 0) || (s3 && d3 == 0))) {
-			ret = new CompressedMatrixBlock();
-			ret.copy(this);
-			return ret;
-		}
-
-		if(m2 instanceof CompressedMatrixBlock)
-			m2 = ((CompressedMatrixBlock) m2).getUncompressed("Ternary Operator arg2 " + op.fn.getClass().getSimpleName(),
-				op.getNumThreads());
-		if(m3 instanceof CompressedMatrixBlock)
-			m3 = ((CompressedMatrixBlock) m3).getUncompressed("Ternary Operator arg3 " + op.fn.getClass().getSimpleName(),
-				op.getNumThreads());
-
-		if(s2 != s3 && (op.fn instanceof PlusMultiply || op.fn instanceof MinusMultiply)) {
-			// SPECIAL CASE for sparse-dense combinations of common +* and -*
-			BinaryOperator bop = ((ValueFunctionWithConstant) op.fn).setOp2Constant(s2 ? d2 : d3);
-			bop.setNumThreads(op.getNumThreads());
-			ret = CLALibBinaryCellOp.binaryOperationsRight(bop, this, s2 ? m3 : m2, ret);
-		}
-		else {
-			final boolean sparseOutput = evalSparseFormatInMemory(m, n, (s1 ? m * n * (d1 != 0 ? 1 : 0) : getNonZeros()) +
-				Math.min(s2 ? m * n : m2.getNonZeros(), s3 ? m * n : m3.getNonZeros()));
-			ret.reset(m, n, sparseOutput);
-			final MatrixBlock thisUncompressed = getUncompressed("Ternary Operation not supported");
-			LibMatrixTercell.tercellOp(thisUncompressed, m2, m3, ret, op);
-			ret.examSparsity();
-		}
-		return ret;
+		return CLALibTernaryOp.ternaryOperations(this, op, m2, m3, ret);
 	}
 
 	@Override
@@ -973,6 +927,11 @@ public class CompressedMatrixBlock extends MatrixBlock {
 			.getUncompressed(message) : (MatrixBlock) mVal;
 	}
 
+	public static MatrixBlock getUncompressed(MatrixValue mVal, String message, int k) {
+		return isCompressed((MatrixBlock) mVal) ? ((CompressedMatrixBlock) mVal).getUncompressed(message,
+			k) : (MatrixBlock) mVal;
+	}
+
 	public MatrixBlock getUncompressed() {
 		return getUncompressed((String) null);
 	}
@@ -1071,6 +1030,11 @@ public class CompressedMatrixBlock extends MatrixBlock {
 		decompressedVersion = null;
 	}
 
+	public void clearCounts() {
+		for(AColGroup a : _colGroups)
+			a.clear();
+	}
+
 	@Override
 	public DenseBlock getDenseBlock() {
 		throw new DMLCompressionException("Should not get DenseBlock on a compressed Matrix");
@@ -1137,7 +1101,8 @@ public class CompressedMatrixBlock extends MatrixBlock {
 	}
 
 	@Override
-	public void appendRowToSparse(SparseBlock dest, MatrixBlock src, int i, int rowoffset, int coloffset, boolean deep) {
+	public void appendRowToSparse(SparseBlock dest, MatrixBlock src, int i, int rowoffset, int coloffset,
+		boolean deep) {
 		throw new DMLCompressionException("Can't append row to compressed Matrix");
 	}
 
@@ -1187,18 +1152,23 @@ public class CompressedMatrixBlock extends MatrixBlock {
 	}
 
 	@Override
-	public void examSparsity(boolean allowCSR) {
+	public void examSparsity(boolean allowCSR, int k) {
 		// do nothing
 	}
 
 	@Override
-	public void sparseToDense() {
+	public void sparseToDense(int k) {
 		// do nothing
 	}
 
 	@Override
-	public void merge(MatrixBlock that, boolean appendOnly, boolean par, boolean deep) {
-		throw new NotImplementedException();
+	public void denseToSparse(boolean allowCSR, int k){
+		// do nothing
+	}
+
+	@Override
+	public MatrixBlock merge(MatrixBlock that, boolean appendOnly, boolean par, boolean deep) {
+		return CLALibMerge.merge(this, that, appendOnly, par, deep);
 	}
 
 	@Override
@@ -1285,4 +1255,19 @@ public class CompressedMatrixBlock extends MatrixBlock {
 	public void allocateAndResetSparseBlock(boolean clearNNZ, SparseBlock.Type stype) {
 		throw new DMLCompressionException("Invalid to allocate block on a compressed MatrixBlock");
 	}
+
+	@Override
+	public String toString() {
+		StringBuilder sb = new StringBuilder();
+		sb.append("CompressedMatrixBlock:");
+		sb.append("\nCols:" + getNumColumns() + " Rows:" + getNumRows() + " Overlapping: " + isOverlapping() + " nnz: "
+			+ nonZeros);
+		if(_colGroups != null)
+			for(AColGroup cg : _colGroups) {
+				sb.append("\n" + cg);
+			}
+		else
+			sb.append("\nEmptyColGroups");
+		return sb.toString();
+	}
 }
