@@ -26,7 +26,7 @@ import java.util.HashMap;
 import java.util.Map;
 import java.util.concurrent.atomic.AtomicLong;
 
-import org.apache.commons.lang.mutable.MutableBoolean;
+import org.apache.commons.lang3.mutable.MutableBoolean;
 import org.apache.commons.logging.Log;
 import org.apache.commons.logging.LogFactory;
 import org.apache.hadoop.fs.Path;
@@ -73,7 +73,7 @@ import org.apache.sysds.utils.Statistics;
  * to allow Java garbage collection. If other parts of the system continue
  * keep references to the cache block, its eviction will not release any memory.
  */
-public abstract class CacheableData<T extends CacheBlock> extends Data
+public abstract class CacheableData<T extends CacheBlock<?>> extends Data
 {
 	private static final long serialVersionUID = -413810592207212835L;
 
@@ -137,7 +137,7 @@ public abstract class CacheableData<T extends CacheBlock> extends Data
 	/**
 	 * Current state of pinned variables, required for guarded collect.
 	 */
-	private static ThreadLocal<Long> sizePinned = new ThreadLocal<Long>() {
+	private static ThreadLocal<Long> sizePinned = new ThreadLocal<>() {
 		@Override protected Long initialValue() { return 0L; }
 	};
 
@@ -366,7 +366,7 @@ public abstract class CacheableData<T extends CacheBlock> extends Data
 		return getDataCharacteristics().getCols();
 	}
 	
-	public long getBlocksize() {
+	public int getBlocksize() {
 		return getDataCharacteristics().getBlocksize();
 	}
 	
@@ -392,8 +392,12 @@ public abstract class CacheableData<T extends CacheBlock> extends Data
 		if(_fedMapping == null && _metaData instanceof MetaDataFormat){
 			MetaDataFormat mdf = (MetaDataFormat) _metaData;
 			if(mdf.getFileFormat() == FileFormat.FEDERATED){
-				InitFEDInstruction.federateMatrix(
-					this, ReaderWriterFederated.read(_hdfsFileName, mdf.getDataCharacteristics()));
+				if (this instanceof FrameObject)
+					InitFEDInstruction.federateFrame((FrameObject) this,
+						ReaderWriterFederated.read(_hdfsFileName, mdf.getDataCharacteristics()));
+				else
+					InitFEDInstruction.federateMatrix(
+							this, ReaderWriterFederated.read(_hdfsFileName, mdf.getDataCharacteristics()));
 				return true;
 			}
 		}
@@ -438,11 +442,19 @@ public abstract class CacheableData<T extends CacheBlock> extends Data
 		if( _rddHandle != null )
 			rdd.setBackReference(this);
 	}
+
+	public boolean hasRDDHandle() {
+		return _rddHandle != null && _rddHandle.hasBackReference();
+	}
 	
 	public BroadcastObject<T> getBroadcastHandle() {
 		return _bcHandle;
 	}
 
+	public boolean hasBroadcastHandle() {
+		return  _bcHandle != null && _bcHandle.hasBackReference();
+	}
+
 	@SuppressWarnings({ "rawtypes", "unchecked" })
 	public void setBroadcastHandle( BroadcastObject bc ) {
 		//cleanup potential old back reference
@@ -778,9 +790,6 @@ public abstract class CacheableData<T extends CacheBlock> extends Data
 			for (GPUObject gObj : _gpuObjects.values())
 				if (gObj != null) {
 					gObj.clearData(null, DMLScript.EAGER_CUDA_FREE);
-					if (gObj.isLinCached())
-						// set rmVarPending which helps detecting liveness
-						gObj.setrmVarPending(true);
 				}
 		}
 		
@@ -1043,7 +1052,7 @@ public abstract class CacheableData<T extends CacheBlock> extends Data
 		return (_data.getInMemorySize() <= CACHING_THRESHOLD);
 	}
 	
-	public static boolean isBelowCachingThreshold(CacheBlock data) {
+	public static boolean isBelowCachingThreshold(CacheBlock<?> data) {
 		boolean ret;
 		if (OptimizerUtils.isUMMEnabled())
 			ret = UnifiedMemoryManager.getCacheBlockSize(data) <= CACHING_THRESHOLD;
@@ -1271,7 +1280,12 @@ public abstract class CacheableData<T extends CacheBlock> extends Data
 	public boolean isPendingRDDOps() {
 		return isEmpty(true) && _data == null && (_rddHandle != null && _rddHandle.hasBackReference());
 	}
-	
+
+	public boolean isDeviceToHostCopy() {
+		boolean isGpuOP = isEmpty(true) && _data == null && _gpuObjects != null;
+		return isGpuOP && _gpuObjects.values().stream().anyMatch(gobj -> (gobj != null && gobj.isDirty()));
+	}
+
 	protected void setEmpty() {
 		_cacheStatus = CacheStatus.EMPTY;
 	}
