@@ -24,10 +24,26 @@ import java.util.Arrays;
 import java.util.List;
 import java.util.Optional;
 import java.util.concurrent.Future;
+import java.util.concurrent.TimeUnit;
 
+import io.netty.channel.ChannelInboundHandlerAdapter;
+import io.netty.channel.ChannelOutboundHandlerAdapter;
+import io.netty.handler.codec.compression.FastLzFrameDecoder;
+import io.netty.handler.codec.compression.FastLzFrameEncoder;
+import io.netty.handler.codec.compression.JdkZlibDecoder;
+import io.netty.handler.codec.compression.JdkZlibEncoder;
+import io.netty.handler.codec.compression.Lz4FrameDecoder;
+import io.netty.handler.codec.compression.Lz4FrameEncoder;
+import io.netty.handler.codec.compression.LzfDecoder;
+import io.netty.handler.codec.compression.LzfEncoder;
+import io.netty.handler.codec.compression.SnappyFrameDecoder;
+import io.netty.handler.codec.compression.SnappyFrameEncoder;
+import org.apache.commons.lang3.tuple.ImmutablePair;
 import org.apache.commons.lang3.tuple.Pair;
 import org.apache.log4j.Logger;
 import org.apache.sysds.common.Types.ExecType;
+import org.apache.sysds.conf.ConfigurationManager;
+import org.apache.sysds.conf.DMLConfig;
 import org.apache.sysds.hops.fedplanner.FTypes.FPartitioning;
 import org.apache.sysds.hops.fedplanner.FTypes.FType;
 import org.apache.sysds.lops.Lop;
@@ -145,13 +161,14 @@ public class FederationUtils {
 					Lop.OPERAND_DELIMITOR + varOldOut.getName() + Lop.DATATYPE_PREFIX,
 					Lop.OPERAND_DELIMITOR + String.valueOf(outputId) + Lop.DATATYPE_PREFIX);
 			}
-			
+
 			fr[j] = new FederatedRequest(RequestType.EXEC_INST, outputId, (Object) linst[j]);
 		}
 		return fr;
 	}
 
 	public static FederatedRequest callInstruction(String inst, CPOperand varOldOut, long outputId, CPOperand[] varOldIn, long[] varNewIn, ExecType type, boolean rmFedOutputFlag) {
+		boolean isFedInstr = inst.startsWith(ExecType.FED.name() + Lop.OPERAND_DELIMITOR);
 		String linst = InstructionUtils.replaceOperand(inst, 0, type.name());
 		linst = linst.replace(Lop.OPERAND_DELIMITOR+varOldOut.getName()+Lop.DATATYPE_PREFIX, Lop.OPERAND_DELIMITOR+outputId+Lop.DATATYPE_PREFIX);
 		for(int i=0; i<varOldIn.length; i++)
@@ -161,7 +178,7 @@ public class FederationUtils {
 					Lop.OPERAND_DELIMITOR+(varNewIn[i])+Lop.DATATYPE_PREFIX);
 				linst = linst.replace("="+varOldIn[i].getName(), "="+(varNewIn[i])); //parameterized
 			}
-		if(rmFedOutputFlag)
+		if(rmFedOutputFlag && isFedInstr)
 			linst = InstructionUtils.removeFEDOutputFlag(linst);
 		return new FederatedRequest(RequestType.EXEC_INST, outputId, linst);
 	}
@@ -434,8 +451,15 @@ public class FederationUtils {
 
 	public static void waitFor(List<Future<FederatedResponse>> responses) {
 		try {
-			for(Future<FederatedResponse> fr : responses)
-				fr.get();
+			final int timeout = ConfigurationManager.getFederatedTimeout();
+			if(timeout > 0){
+				for(Future<FederatedResponse> fr : responses)
+					fr.get(timeout, TimeUnit.SECONDS);
+			}
+			else {
+				for(Future<FederatedResponse> fr : responses)
+					fr.get();
+			}
 		}
 		catch(Exception ex) {
 			throw new DMLRuntimeException(ex);
@@ -486,7 +510,19 @@ public class FederationUtils {
 			throw new DMLRuntimeException(ex);
 		}
 	}
-
+	
+	public static boolean aggBooleanScalar(Future<FederatedResponse>[] tmp) {
+		boolean ret = false;
+		try {
+			for( Future<FederatedResponse> fr : tmp )
+				ret |= ((ScalarObject)fr.get().getData()[0]).getBooleanValue();
+		}
+		catch (Exception e) {
+			throw new DMLRuntimeException(e);
+		}
+		return ret;
+	}
+	
 	public static MatrixBlock aggMatrix(AggregateUnaryOperator aop, Future<FederatedResponse>[] ffr, FederationMap map) {
 		if (aop.isRowAggregate() && map.getType() == FType.ROW)
 			return bind(ffr, false);
@@ -538,7 +574,13 @@ public class FederationUtils {
 	public static MatrixBlock bindResponses(List<Pair<FederatedRange, Future<FederatedResponse>>> readResponses, long[] dims)
 		throws Exception
 	{
-		MatrixBlock ret = new MatrixBlock((int) dims[0], (int) dims[1], false);
+		long totalNNZ = 0;
+		for(Pair<FederatedRange, Future<FederatedResponse>> readResponse : readResponses) {
+			FederatedResponse response = readResponse.getRight().get();
+			MatrixBlock multRes = (MatrixBlock) response.getData()[0];
+			totalNNZ += multRes.getNonZeros();
+		}
+		MatrixBlock ret = new MatrixBlock((int) dims[0], (int) dims[1], MatrixBlock.evalSparseFormatInMemory(dims[0], dims[1], totalNNZ));
 		for(Pair<FederatedRange, Future<FederatedResponse>> readResponse : readResponses) {
 			FederatedRange range = readResponse.getLeft();
 			FederatedResponse response = readResponse.getRight().get();
@@ -547,8 +589,8 @@ public class FederationUtils {
 			int[] endDimsInt = range.getEndDimsInt();
 			MatrixBlock multRes = (MatrixBlock) response.getData()[0];
 			ret.copy(beginDimsInt[0], endDimsInt[0] - 1, beginDimsInt[1], endDimsInt[1] - 1, multRes, false);
-			ret.setNonZeros(ret.getNonZeros() + multRes.getNonZeros());
 		}
+		ret.setNonZeros(totalNNZ);
 		return ret;
 	}
 
@@ -570,4 +612,43 @@ public class FederationUtils {
 		return new ObjectDecoder(Integer.MAX_VALUE,
 			ClassResolvers.weakCachingResolver(ClassLoader.getSystemClassLoader()));
 	}
+
+	public static Optional<ChannelOutboundHandlerAdapter> compressionEncoder() {
+		return compressionStrategy().map(strategy -> strategy.right);
+	}
+
+	public static Optional<ChannelInboundHandlerAdapter> compressionDecoder() {
+		return compressionStrategy().map(strategy -> strategy.left);
+	}
+
+	public static Optional<ImmutablePair<ChannelInboundHandlerAdapter, ChannelOutboundHandlerAdapter>> compressionStrategy() {
+		String strategy = ConfigurationManager.getDMLConfig().getTextValue(DMLConfig.FEDERATED_COMPRESSION).toLowerCase();
+		switch (strategy) {
+			case "none":
+				return Optional.empty();
+			case "zlib":
+				return Optional.of(new ImmutablePair<>(new JdkZlibDecoder(), new JdkZlibEncoder()));
+			case "snappy":
+				return Optional.of(new ImmutablePair<>(new SnappyFrameDecoder(), new SnappyFrameEncoder()));
+			case "fastlz":
+				return Optional.of(new ImmutablePair<>(new FastLzFrameDecoder(), new FastLzFrameEncoder()));
+			case "lz4":
+				return Optional.of(new ImmutablePair<>(new Lz4FrameDecoder(), new Lz4FrameEncoder()));
+			case "lzf":
+				return Optional.of(new ImmutablePair<>(new LzfDecoder(), new LzfEncoder()));
+			default:
+				throw new IllegalArgumentException("Invalid federated compression strategy: " + strategy);
+		}
+	}
+
+	public static long sumNonZeros(Future<FederatedResponse>[] responses) {
+		long nnz = 0;
+		try {
+			for( Future<FederatedResponse> r : responses)
+				nnz += (Long)r.get().getData()[0];
+			return nnz;
+		}
+		catch(Exception ex) { }
+		return -1;
+	}
 }
