@@ -25,16 +25,19 @@ import org.apache.spark.api.java.JavaSparkContext;
 import org.apache.spark.api.java.function.Function;
 import org.apache.spark.api.java.function.Function2;
 import org.apache.spark.api.java.function.PairFlatMapFunction;
+import org.apache.spark.api.java.function.VoidFunction;
 import org.apache.spark.storage.StorageLevel;
 import org.apache.sysds.hops.OptimizerUtils;
 import org.apache.sysds.lops.Checkpoint;
 import org.apache.sysds.runtime.controlprogram.caching.MatrixObject;
+import org.apache.sysds.runtime.controlprogram.context.ExecutionContext;
 import org.apache.sysds.runtime.controlprogram.context.SparkExecutionContext;
 import org.apache.sysds.runtime.controlprogram.parfor.stat.InfrastructureAnalyzer;
 import org.apache.sysds.runtime.data.IndexedTensorBlock;
 import org.apache.sysds.runtime.data.BasicTensorBlock;
 import org.apache.sysds.runtime.data.TensorBlock;
 import org.apache.sysds.runtime.data.TensorIndexes;
+import org.apache.sysds.runtime.frame.data.FrameBlock;
 import org.apache.sysds.runtime.instructions.spark.data.IndexedMatrixValue;
 import org.apache.sysds.runtime.instructions.spark.functions.CopyBinaryCellFunction;
 import org.apache.sysds.runtime.instructions.spark.functions.CopyMatrixBlockFunction;
@@ -43,7 +46,6 @@ import org.apache.sysds.runtime.instructions.spark.functions.CopyTensorBlockFunc
 import org.apache.sysds.runtime.instructions.spark.functions.CopyTensorBlockPairFunction;
 import org.apache.sysds.runtime.instructions.spark.functions.FilterNonEmptyBlocksFunction;
 import org.apache.sysds.runtime.instructions.spark.functions.RecomputeNnzFunction;
-import org.apache.sysds.runtime.matrix.data.FrameBlock;
 import org.apache.sysds.runtime.matrix.data.MatrixBlock;
 import org.apache.sysds.runtime.matrix.data.MatrixCell;
 import org.apache.sysds.runtime.matrix.data.MatrixIndexes;
@@ -197,6 +199,12 @@ public class SparkUtils
 		else //requires key access, so use mappartitions
 			return in.mapPartitionsToPair(new CopyTensorBlockPairFunction(deep), true);
 	}
+	
+	public static void checkSparsity(String varname, ExecutionContext ec) {
+		SparkExecutionContext sec = (SparkExecutionContext) ec;
+		sec.getBinaryMatrixBlockRDDHandleForVariable(varname)
+			.foreach(new CheckSparsityFunction());
+	}
 
 	// This returns RDD with identifier as well as location
 	public static String getStartLineFromSparkDebugInfo(String line) {
@@ -285,6 +293,17 @@ public class SparkUtils
 			mo.acquireReadAndRelease();
 	}
 	
+	private static class CheckSparsityFunction implements VoidFunction<Tuple2<MatrixIndexes,MatrixBlock>>
+	{
+		private static final long serialVersionUID = 4150132775681848807L;
+
+		@Override
+		public void call(Tuple2<MatrixIndexes, MatrixBlock> arg) throws Exception {
+			arg._2.checkNonZeros();
+			arg._2.checkSparseRows();
+		}
+	}
+	
 	private static class AnalyzeCellDataCharacteristics implements Function<Tuple2<MatrixIndexes,MatrixCell>, DataCharacteristics>
 	{
 		private static final long serialVersionUID = 8899395272683723008L;
