@@ -19,43 +19,58 @@
 
 package org.apache.sysds.runtime.transform.encode;
 
-import org.apache.commons.lang.ArrayUtils;
-import org.apache.sysds.api.DMLScript;
-import org.apache.sysds.common.Types.ValueType;
-import org.apache.sysds.runtime.DMLRuntimeException;
-import org.apache.sysds.runtime.matrix.data.FrameBlock;
-import org.apache.sysds.runtime.transform.TfUtils.TfMethod;
-import org.apache.sysds.runtime.transform.encode.ColumnEncoder.EncoderType;
-import org.apache.sysds.runtime.transform.meta.TfMetaUtils;
-import org.apache.sysds.runtime.util.UtilFunctions;
-import org.apache.sysds.utils.Statistics;
-import org.apache.wink.json4j.JSONArray;
-import org.apache.wink.json4j.JSONObject;
+import static org.apache.sysds.runtime.util.CollectionUtils.except;
+import static org.apache.sysds.runtime.util.CollectionUtils.intersect;
+import static org.apache.sysds.runtime.util.CollectionUtils.unionDistinct;
 
 import java.util.ArrayList;
 import java.util.Arrays;
 import java.util.HashMap;
 import java.util.List;
 import java.util.Map.Entry;
+import java.util.Objects;
 
-import static org.apache.sysds.runtime.util.CollectionUtils.except;
-import static org.apache.sysds.runtime.util.CollectionUtils.unionDistinct;
+import org.apache.commons.lang3.ArrayUtils;
+import org.apache.commons.logging.Log;
+import org.apache.commons.logging.LogFactory;
+import org.apache.sysds.api.DMLScript;
+import org.apache.sysds.common.Types.ValueType;
+import org.apache.sysds.runtime.DMLRuntimeException;
+import org.apache.sysds.runtime.frame.data.FrameBlock;
+import org.apache.sysds.runtime.matrix.data.MatrixBlock;
+import org.apache.sysds.runtime.transform.TfUtils.TfMethod;
+import org.apache.sysds.runtime.transform.encode.ColumnEncoder.EncoderType;
+import org.apache.sysds.runtime.transform.meta.TfMetaUtils;
+import org.apache.sysds.runtime.util.UtilFunctions;
+import org.apache.sysds.utils.stats.TransformStatistics;
+import org.apache.wink.json4j.JSONArray;
+import org.apache.wink.json4j.JSONObject;
 
-public class EncoderFactory {
+public interface EncoderFactory {
+	final static Log LOG = LogFactory.getLog(EncoderFactory.class.getName());
+
+	public static MultiColumnEncoder createEncoder(String spec, int clen) {
+		return createEncoder(spec, null, clen, null, null, -1, -1);
+	}
 
 	public static MultiColumnEncoder createEncoder(String spec, String[] colnames, int clen, FrameBlock meta) {
-		return createEncoder(spec, colnames, UtilFunctions.nCopies(clen, ValueType.STRING), meta);
+		return createEncoder(spec, colnames, clen, meta, null, -1, -1);
 	}
 
 	public static MultiColumnEncoder createEncoder(String spec, String[] colnames, int clen, FrameBlock meta,
 		int minCol, int maxCol) {
-		return createEncoder(spec, colnames, UtilFunctions.nCopies(clen, ValueType.STRING), meta, minCol, maxCol);
+		return createEncoder(spec, colnames, clen, meta, null, minCol, maxCol);
 	}
 
 	public static MultiColumnEncoder createEncoder(String spec, String[] colnames, ValueType[] schema, int clen,
 		FrameBlock meta) {
+		return createEncoder(spec, colnames, clen, meta);
+	}
+
+	public static MultiColumnEncoder createEncoder(String spec, String[] colnames, ValueType[] schema, int clen,
+												   FrameBlock meta, MatrixBlock embeddings) {
 		ValueType[] lschema = (schema == null) ? UtilFunctions.nCopies(clen, ValueType.STRING) : schema;
-		return createEncoder(spec, colnames, lschema, meta);
+		return createEncoder(spec, colnames, lschema, meta, embeddings);
 	}
 
 	public static MultiColumnEncoder createEncoder(String spec, String[] colnames, ValueType[] schema,
@@ -65,8 +80,24 @@ public class EncoderFactory {
 
 	public static MultiColumnEncoder createEncoder(String spec, String[] colnames, ValueType[] schema, FrameBlock meta,
 		int minCol, int maxCol) {
+		return createEncoder(spec, colnames, schema.length, meta, null, minCol, maxCol);
+	}
+
+	public static MultiColumnEncoder createEncoder(String spec, String[] colnames, int clen, FrameBlock meta, MatrixBlock embeddings) {
+		return createEncoder(spec, colnames, UtilFunctions.nCopies(clen, ValueType.STRING), meta, embeddings);
+	}
+
+	public static MultiColumnEncoder createEncoder(String spec, String[] colnames, ValueType[] schema,
+												   FrameBlock meta, MatrixBlock embeddings) {
+		return createEncoder(spec, colnames, schema.length, meta, embeddings, -1, -1);
+	}
+
+	public static MultiColumnEncoder createEncoder(String spec, String[] colnames, int clen, FrameBlock meta,
+		MatrixBlock embeddings, int minCol, int maxCol) {
+
+
 		MultiColumnEncoder encoder;
-		int clen = schema.length;
+		// int clen = schema.length;
 
 		try {
 			// parse transform specification
@@ -74,6 +105,7 @@ public class EncoderFactory {
 			List<ColumnEncoderComposite> lencoders = new ArrayList<>();
 			HashMap<Integer, List<ColumnEncoder>> colEncoders = new HashMap<>();
 			boolean ids = jSpec.containsKey("ids") && jSpec.getBoolean("ids");
+			TfMetaUtils.checkValidEncoders(jSpec);
 
 			// prepare basic id lists (recode, feature hash, dummycode, pass-through)
 			List<Integer> rcIDs = Arrays.asList(ArrayUtils
@@ -83,33 +115,44 @@ public class EncoderFactory {
 			List<Integer> dcIDs = Arrays.asList(ArrayUtils
 				.toObject(TfMetaUtils.parseJsonIDList(jSpec, colnames, TfMethod.DUMMYCODE.toString(), minCol, maxCol)));
 			List<Integer> binIDs = TfMetaUtils.parseBinningColIDs(jSpec, colnames, minCol, maxCol);
-			// note: any dummycode column requires recode as preparation, unless it follows binning
-			rcIDs = except(unionDistinct(rcIDs, except(dcIDs, binIDs)), haIDs);
-			List<Integer> ptIDs = except(except(UtilFunctions.getSeqList(1, clen, 1), unionDistinct(rcIDs, haIDs)),
-				binIDs);
+			List<Integer> weIDs = Arrays.asList(ArrayUtils
+					.toObject(TfMetaUtils.parseJsonIDList(jSpec, colnames, TfMethod.WORD_EMBEDDING.toString(), minCol, maxCol)));
+
+			//check if user passed an embeddings matrix
+			if(!weIDs.isEmpty() && embeddings == null)
+				throw new DMLRuntimeException("Missing argument Embeddings Matrix for transform [" + TfMethod.WORD_EMBEDDING + "]");
+
+			// NOTE: any dummycode column requires recode as preparation, unless the dummycode
+			// column follows binning or feature hashing
+			rcIDs = unionDistinct(rcIDs, except(except(dcIDs, binIDs), haIDs));
+			// NOTE: Word Embeddings requires recode as preparation
+			//rcIDs = unionDistinct(rcIDs, weIDs);
+			// Error out if the first level encoders have overlaps
+			if (intersect(rcIDs, binIDs, haIDs, weIDs))
+				throw new DMLRuntimeException("More than one encoders (recode, binning, hashing, word_embedding) on one column is not allowed");
+
+			List<Integer> ptIDs = except(except(except(UtilFunctions.getSeqList(1, clen, 1), unionDistinct(rcIDs, haIDs)),
+				binIDs), weIDs);
 			List<Integer> oIDs = Arrays.asList(ArrayUtils
 				.toObject(TfMetaUtils.parseJsonIDList(jSpec, colnames, TfMethod.OMIT.toString(), minCol, maxCol)));
 			List<Integer> mvIDs = Arrays.asList(ArrayUtils.toObject(
 				TfMetaUtils.parseJsonObjectIDList(jSpec, colnames, TfMethod.IMPUTE.toString(), minCol, maxCol)));
+			List<Integer> udfIDs = TfMetaUtils.parseUDFColIDs(jSpec, colnames, minCol, maxCol);
+
 
 			// create individual encoders
-			if(!rcIDs.isEmpty()) {
-				for(Integer id : rcIDs) {
-					ColumnEncoderRecode ra = new ColumnEncoderRecode(id);
-					addEncoderToMap(ra, colEncoders);
-				}
-			}
-			if(!haIDs.isEmpty()) {
-				for(Integer id : haIDs) {
-					ColumnEncoderFeatureHash ha = new ColumnEncoderFeatureHash(id, TfMetaUtils.getK(jSpec));
-					addEncoderToMap(ha, colEncoders);
-				}
-			}
+			if(!rcIDs.isEmpty())
+				for(Integer id : rcIDs)
+					addEncoderToMap(new ColumnEncoderRecode(id), colEncoders);
+			if(!haIDs.isEmpty())
+				for(Integer id : haIDs)
+					addEncoderToMap(new ColumnEncoderFeatureHash(id, TfMetaUtils.getK(jSpec)), colEncoders);
 			if(!ptIDs.isEmpty())
-				for(Integer id : ptIDs) {
-					ColumnEncoderPassThrough pt = new ColumnEncoderPassThrough(id);
-					addEncoderToMap(pt, colEncoders);
-				}
+				for(Integer id : ptIDs)
+					addEncoderToMap(new ColumnEncoderPassThrough(id), colEncoders);
+			if(!weIDs.isEmpty())
+				for(Integer id : weIDs)
+					addEncoderToMap(new ColumnEncoderWordEmbedding(id), colEncoders);
 			if(!binIDs.isEmpty())
 				for(Object o : (JSONArray) jSpec.get(TfMethod.BIN.toString())) {
 					JSONObject colspec = (JSONObject) o;
@@ -117,43 +160,58 @@ public class EncoderFactory {
 					int id = TfMetaUtils.parseJsonObjectID(colspec, colnames, minCol, maxCol, ids);
 					if(id <= 0)
 						continue;
-					ColumnEncoderBin bin = new ColumnEncoderBin(id, numBins);
+					String method = colspec.get("method").toString().toUpperCase();
+					ColumnEncoderBin.BinMethod binMethod;
+					if ("EQUI-WIDTH".equals(method))
+						binMethod = ColumnEncoderBin.BinMethod.EQUI_WIDTH;
+					else if ("EQUI-HEIGHT".equals(method))
+						binMethod = ColumnEncoderBin.BinMethod.EQUI_HEIGHT;
+					else if ("EQUI-HEIGHT-APPROX".equals(method))
+						binMethod = ColumnEncoderBin.BinMethod.EQUI_HEIGHT_APPROX;
+					else
+						throw new DMLRuntimeException("Unsupported binning method: " + method);
+					ColumnEncoderBin bin = new ColumnEncoderBin(id, numBins, binMethod);
 					addEncoderToMap(bin, colEncoders);
 				}
 			if(!dcIDs.isEmpty())
-				for(Integer id : dcIDs) {
-					ColumnEncoderDummycode dc = new ColumnEncoderDummycode(id);
-					addEncoderToMap(dc, colEncoders);
-				}
+				for(Integer id : dcIDs)
+					addEncoderToMap(new ColumnEncoderDummycode(id), colEncoders);
+			if(!udfIDs.isEmpty()) {
+				String name = jSpec.getJSONObject("udf").getString("name");
+				for(Integer id : udfIDs)
+					addEncoderToMap(new ColumnEncoderUDF(id, name), colEncoders);
+			}
+			
 			// create composite decoder of all created encoders
 			for(Entry<Integer, List<ColumnEncoder>> listEntry : colEncoders.entrySet()) {
 				if(DMLScript.STATISTICS)
-					Statistics.incTransformEncoderCount(listEntry.getValue().size());
+					TransformStatistics.incEncoderCount(listEntry.getValue().size());
 				lencoders.add(new ColumnEncoderComposite(listEntry.getValue()));
 			}
 			encoder = new MultiColumnEncoder(lencoders);
 			if(!oIDs.isEmpty()) {
-				encoder.addReplaceLegacyEncoder(new EncoderOmit(jSpec, colnames, schema.length, minCol, maxCol));
+				encoder.addReplaceLegacyEncoder(new EncoderOmit(jSpec, colnames, clen, minCol, maxCol));
 				if(DMLScript.STATISTICS)
-					Statistics.incTransformEncoderCount(1);
+					TransformStatistics.incEncoderCount(1);
 			}
 			if(!mvIDs.isEmpty()) {
-				EncoderMVImpute ma = new EncoderMVImpute(jSpec, colnames, schema.length, minCol, maxCol);
+				EncoderMVImpute ma = new EncoderMVImpute(jSpec, colnames, clen, minCol, maxCol);
 				ma.initRecodeIDList(rcIDs);
 				encoder.addReplaceLegacyEncoder(ma);
 				if(DMLScript.STATISTICS)
-					Statistics.incTransformEncoderCount(1);
+					TransformStatistics.incEncoderCount(1);
 			}
 
 			// initialize meta data w/ robustness for superset of cols
 			if(meta != null) {
 				String[] colnames2 = meta.getColumnNames();
+
 				if(!TfMetaUtils.isIDSpec(jSpec) && colnames != null && colnames2 != null &&
-					!ArrayUtils.isEquals(colnames, colnames2)) {
+					!Objects.deepEquals(colnames, colnames2)) {
 					HashMap<String, Integer> colPos = getColumnPositions(colnames2);
 					// create temporary meta frame block w/ shallow column copy
 					FrameBlock meta2 = new FrameBlock(meta.getSchema(), colnames2);
-					meta2.setNumRows(meta.getNumRows());
+
 					for(int i = 0; i < colnames.length; i++) {
 						if(!colPos.containsKey(colnames[i])) {
 							throw new DMLRuntimeException("Column name not found in meta data: " + colnames[i]
@@ -167,6 +225,9 @@ public class EncoderFactory {
 				}
 				encoder.initMetaData(meta);
 			}
+			//initialize embeddings matrix block in the encoders in case word embedding transform is used
+			if(!weIDs.isEmpty())
+				encoder.initEmbeddings(embeddings);
 		}
 		catch(Exception ex) {
 			throw new DMLRuntimeException(ex);
@@ -182,6 +243,8 @@ public class EncoderFactory {
 	}
 
 	public static int getEncoderType(ColumnEncoder columnEncoder) {
+		//TODO replace with columnEncoder.getType().ordinal
+		//(which requires a cleanup of all type handling)
 		if(columnEncoder instanceof ColumnEncoderBin)
 			return EncoderType.Bin.ordinal();
 		else if(columnEncoder instanceof ColumnEncoderDummycode)
@@ -192,6 +255,8 @@ public class EncoderFactory {
 			return EncoderType.PassThrough.ordinal();
 		else if(columnEncoder instanceof ColumnEncoderRecode)
 			return EncoderType.Recode.ordinal();
+		else if(columnEncoder instanceof ColumnEncoderWordEmbedding)
+			return EncoderType.WordEmbedding.ordinal();
 		throw new DMLRuntimeException("Unsupported encoder type: " + columnEncoder.getClass().getCanonicalName());
 	}
 
@@ -208,6 +273,8 @@ public class EncoderFactory {
 				return new ColumnEncoderPassThrough();
 			case Recode:
 				return new ColumnEncoderRecode();
+			case WordEmbedding:
+				return new ColumnEncoderWordEmbedding();
 			default:
 				throw new DMLRuntimeException("Unsupported encoder type: " + etype);
 		}
@@ -219,5 +286,4 @@ public class EncoderFactory {
 			ret.put(colnames[i], i);
 		return ret;
 	}
-
 }
