@@ -33,15 +33,20 @@ import java.io.IOException;
 import java.util.ArrayList;
 import java.util.Arrays;
 import java.util.Collection;
+import java.util.Random;
 
 import org.apache.commons.logging.Log;
 import org.apache.commons.logging.LogFactory;
+import org.apache.sysds.runtime.compress.CompressedMatrixBlock;
 import org.apache.sysds.runtime.compress.DMLCompressionException;
+import org.apache.sysds.runtime.compress.colgroup.AOffsetsGroup;
 import org.apache.sysds.runtime.compress.colgroup.offset.AIterator;
 import org.apache.sysds.runtime.compress.colgroup.offset.AOffset;
+import org.apache.sysds.runtime.compress.colgroup.offset.AOffset.OffsetSliceInfo;
 import org.apache.sysds.runtime.compress.colgroup.offset.AOffsetIterator;
 import org.apache.sysds.runtime.compress.colgroup.offset.OffsetByte;
 import org.apache.sysds.runtime.compress.colgroup.offset.OffsetChar;
+import org.apache.sysds.runtime.compress.colgroup.offset.OffsetEmpty;
 import org.apache.sysds.runtime.compress.colgroup.offset.OffsetFactory;
 import org.apache.sysds.runtime.compress.colgroup.offset.OffsetFactory.OFF_TYPE;
 import org.apache.sysds.runtime.compress.colgroup.offset.OffsetSingle;
@@ -66,6 +71,7 @@ public class OffsetTests {
 		ArrayList<Object[]> tests = new ArrayList<>();
 		// It is assumed that the input is in sorted order, all values are positive and there are no duplicates.
 		for(OFF_TYPE t : OFF_TYPE.values()) {
+			tests.add(new Object[] {new int[] {}, t});
 			tests.add(new Object[] {new int[] {1, 2}, t});
 			tests.add(new Object[] {new int[] {2, 142}, t});
 			tests.add(new Object[] {new int[] {142, 421}, t});
@@ -76,15 +82,13 @@ public class OffsetTests {
 			tests.add(new Object[] {new int[] {0}, t});
 			tests.add(new Object[] {new int[] {500}, t});
 			tests.add(new Object[] {new int[] {1442}, t});
-			tests.add(new Object[] {new int[] {Character.MAX_VALUE, ((int) Character.MAX_VALUE) + 1}, t});
-			tests.add(new Object[] {new int[] {Character.MAX_VALUE, ((int) Character.MAX_VALUE) * 2}, t});
+			tests.add(new Object[] {new int[] {Character.MAX_VALUE, Character.MAX_VALUE + 1}, t});
 			tests.add(new Object[] {new int[] {0, 256}, t});
 			tests.add(new Object[] {new int[] {0, 254}, t});
 			tests.add(new Object[] {new int[] {0, Character.MAX_VALUE}, t});
-			tests.add(new Object[] {new int[] {0, Character.MAX_VALUE, ((int) Character.MAX_VALUE) * 2}, t});
 			tests.add(new Object[] {new int[] {2, Character.MAX_VALUE + 2}, t});
-			tests.add(new Object[] {new int[] {0, ((int) Character.MAX_VALUE) + 1}, t});
-			tests.add(new Object[] {new int[] {0, ((int) Character.MAX_VALUE) - 1}, t});
+			tests.add(new Object[] {new int[] {0, Character.MAX_VALUE + 1}, t});
+			tests.add(new Object[] {new int[] {0, Character.MAX_VALUE - 1}, t});
 			tests.add(new Object[] {new int[] {0, 256 * 2}, t});
 			tests.add(new Object[] {new int[] {0, 255 * 2}, t});
 			tests.add(new Object[] {new int[] {0, 254 * 2}, t});
@@ -96,6 +100,8 @@ public class OffsetTests {
 			tests.add(new Object[] {new int[] {0, 255 * 3}, t});
 			tests.add(new Object[] {new int[] {0, 255 * 4}, t});
 			tests.add(new Object[] {new int[] {0, 256 * 3}, t});
+			tests.add(new Object[] {new int[] {0, 100, 200, 300, 400, 500, 600}, t});
+			tests.add(new Object[] {new int[] {0, 200, 400, 600, 800, 1000, 1200}, t});
 			tests.add(new Object[] {new int[] {255 * 3, 255 * 5}, t});
 			tests.add(new Object[] {new int[] {1000000, 1000000 + 255 * 5}, t});
 			tests.add(new Object[] {new int[] {100000000, 100000000 + 255 * 5}, t});
@@ -104,11 +110,39 @@ public class OffsetTests {
 			tests.add(new Object[] {new int[] {0, 1, 2, 3, 4, 5}, t});
 			tests.add(new Object[] {new int[] {2458248, 2458249, 2458253, 2458254, 2458256, 2458257, 2458258, 2458262,
 				2458264, 2458266, 2458267, 2458271, 2458272, 2458275, 2458276, 2458281}, t});
+
+			tests.add(new Object[] {gen(100, 100, 1), t});
+			for(int i = 0; i < 10; i++) {
+				tests.add(new Object[] {gen(100, 200, i), t});
+				tests.add(new Object[] {gen(100, 250, i + 10230), t});
+				tests.add(new Object[] {gen(100, 4, i + 120), t});
+				tests.add(new Object[] {gen(100, 350, i + 1030), t});
+				tests.add(new Object[] {gen(30, 1000, i + 101420), t});
+				tests.add(new Object[] {gen(30, 3000, i + 101420), t});
+
+			}
 		}
+		tests.add(new Object[] {new int[] {Character.MAX_VALUE, Character.MAX_VALUE * 2}, OFF_TYPE.CHAR});
+		tests.add(new Object[] {new int[] {0, Character.MAX_VALUE, Character.MAX_VALUE * 2}, OFF_TYPE.CHAR});
+		tests.add(new Object[] {new int[] {1, Character.MAX_VALUE * 2 + 3, Character.MAX_VALUE * 4 + 4,
+			Character.MAX_VALUE * 16 + 4}, OFF_TYPE.CHAR});
 		return tests;
 	}
 
+	private static int[] gen(int i, int j, int seed) {
+		int[] a = new int[i];
+		Random r = new Random(seed);
+		int o = r.nextInt(j);
+		a[0] = o;
+		for(int k = 1; k < i; k++) {
+			o += r.nextInt(j) + 1;
+			a[k] = o;
+		}
+		return a;
+	}
+
 	public OffsetTests(int[] data, OFF_TYPE type) {
+		CompressedMatrixBlock.debug = true;
 		this.data = data;
 		this.type = type;
 		this.o = OffsetTestUtil.getOffset(data, type);
@@ -189,6 +223,15 @@ public class OffsetTests {
 		assertEquals(data.length, o.getSize());
 	}
 
+	@Test
+	public void offsetSet() {
+		AIterator a = o.getIterator();
+		if(a != null) {
+			a.setOff(324);
+			assertTrue(a.value() == 324);
+		}
+	}
+
 	@Test
 	public void testOnDiskSizeInBytes() {
 		try {
@@ -215,28 +258,31 @@ public class OffsetTests {
 			final long inMemorySize = o.getInMemorySize();
 			long estimatedSize;
 
-			switch(type) {
-				case SINGLE_OFFSET:
-					if(data.length == 1) {
-						estimatedSize = OffsetSingle.estimateInMemorySize();
+			if(data.length == 0)
+				estimatedSize = OffsetEmpty.estimateInMemorySize();
+			else if(data.length == 1)
+				estimatedSize = OffsetSingle.estimateInMemorySize();
+			else if(data.length == 2)
+				estimatedSize = OffsetTwo.estimateInMemorySize();
+			else {
+
+				switch(type) {
+					case BYTE:
+					case UBYTE:
+						final int correctionByte = OffsetFactory.correctionByte(data[data.length - 1] - data[0],
+							data.length);
+						estimatedSize = OffsetByte.estimateInMemorySize(data.length + correctionByte);
 						break;
-					}
-				case TWO_OFFSET:
-					if(data.length == 2) {
-						estimatedSize = OffsetTwo.estimateInMemorySize();
+					case CHAR:
+						final int correctionChar = OffsetFactory.correctionChar(data[data.length - 1] - data[0],
+							data.length);
+						estimatedSize = OffsetChar.estimateInMemorySize(data.length + correctionChar);
 						break;
-					}
-				case BYTE:
-					final int correctionByte = OffsetFactory.correctionByte(data[data.length - 1] - data[0], data.length);
-					estimatedSize = OffsetByte.estimateInMemorySize(data.length + correctionByte);
-					break;
-				case CHAR:
-					final int correctionChar = OffsetFactory.correctionChar(data[data.length - 1] - data[0], data.length);
-					estimatedSize = OffsetChar.estimateInMemorySize(data.length + correctionChar);
-					break;
-				default:
-					throw new DMLCompressionException("Unknown input");
+					default:
+						throw new DMLCompressionException("Unknown input");
+				}
 			}
+
 			if(!(inMemorySize <= estimatedSize + sizeTolerance)) {
 				fail("in memory size: " + inMemorySize + " is not smaller than estimate: " + estimatedSize
 					+ " with tolerance " + sizeTolerance + "\nEncoded:" + o + "\nData:" + Arrays.toString(data));
@@ -251,7 +297,8 @@ public class OffsetTests {
 	@Test
 	public void testSkipToContainedIndex() {
 		try {
-			assertEquals(data[data.length - 1], o.getIterator().skipTo(data[data.length - 1]));
+			if(data.length > 0)
+				assertEquals(data[data.length - 1], o.getIterator().skipTo(data[data.length - 1]));
 		}
 		catch(Exception e) {
 			e.printStackTrace();
@@ -262,7 +309,8 @@ public class OffsetTests {
 	@Test
 	public void testSkipToContainedIndexPlusOne() {
 		try {
-			assertNotEquals(data[data.length - 1] + 1, o.getIterator().skipTo(data[data.length - 1]));
+			if(data.length > 0)
+				assertNotEquals(data[data.length - 1] + 1, o.getIterator().skipTo(data[data.length - 1]));
 		}
 		catch(Exception e) {
 			e.printStackTrace();
@@ -285,9 +333,11 @@ public class OffsetTests {
 	@Test
 	public void testSkipToContainedIndexMinusOne() {
 		try {
-			int v = data[data.length - 1];
-			int maxDiff = 1;
-			assertTrue(v <= o.getIterator().skipTo(v - 1) + maxDiff);
+			if(data.length > 0) {
+				int v = data[data.length - 1];
+				int maxDiff = 1;
+				assertTrue(v <= o.getIterator().skipTo(v - 1) + maxDiff);
+			}
 		}
 		catch(Exception e) {
 			e.printStackTrace();
@@ -298,9 +348,11 @@ public class OffsetTests {
 	@Test
 	public void testSkipToContainedIndexMinusN() {
 		try {
-			int v = data[data.length - 1];
-			int maxDiff = 142;
-			assertTrue(v <= o.getIterator().skipTo(v - 1) + maxDiff);
+			if(data.length > 0) {
+				int v = data[data.length - 1];
+				int maxDiff = 142;
+				assertTrue(v <= o.getIterator().skipTo(v - 1) + maxDiff);
+			}
 		}
 		catch(Exception e) {
 			e.printStackTrace();
@@ -310,17 +362,29 @@ public class OffsetTests {
 
 	@Test
 	public void testToString() {
-		String os = o.toString();
-		os = os.substring(os.indexOf("["), os.length());
-		String vs = Arrays.toString(data);
-		if(!os.equals(vs)) {
-			fail("The two array string are not equivalent with " + type + "\n" + os + " : " + vs);
+		try {
+			String os = o.toString();
+			if(data.length > 0) {
+				os = os.substring(os.indexOf("["), os.length());
+				String vs = Arrays.toString(data);
+				if(!os.equals(vs)) {
+					fail("The two array string are not equivalent with " + type + "\n" + os + " : " + vs);
+				}
+			}
+			else {
+				assertTrue(os.contains("Empty"));
+			}
+		}
+		catch(Exception e) {
+			e.printStackTrace();
 		}
 	}
 
 	@Test
 	public void testIsNotOverFirstDataPoint() {
-		assertFalse(o.getIterator().isNotOver(data[0]));
+
+		if(data.length > 0)
+			assertFalse(o.getIterator().isNotOver(data[0]));
 	}
 
 	@Test
@@ -355,7 +419,9 @@ public class OffsetTests {
 
 	@Test
 	public void testGetDataIndexOnInit() {
-		assertTrue(o.getIterator().getDataIndex() == 0);
+
+		if(data.length > 0)
+			assertTrue(o.getIterator().getDataIndex() == 0);
 	}
 
 	@Test
@@ -418,12 +484,13 @@ public class OffsetTests {
 
 	@Test
 	public void testCloneIterator() {
-		assertTrue(o.getIterator().clone().equals(o.getIterator()));
+		if(o.getIterator() != null)
+			assertTrue(o.getIterator().clone().equals(o.getIterator()));
 	}
 
 	@Test
 	public void testCloneIteratorNext() {
-		if(data.length > 1 || type == OFF_TYPE.SINGLE_OFFSET) {
+		if(data.length > 1) {
 
 			AIterator a = o.getIterator().clone();
 			AIterator b = o.getIterator();
@@ -436,8 +503,7 @@ public class OffsetTests {
 
 	@Test
 	public void testCloneIteratorOffsetNext() {
-		if(data.length > 1 || type == OFF_TYPE.SINGLE_OFFSET) {
-
+		if(data.length > 1) {
 			AOffsetIterator a = o.getOffsetIterator();
 			AOffsetIterator b = o.getOffsetIterator();
 			a.next();
@@ -446,32 +512,340 @@ public class OffsetTests {
 		}
 	}
 
-	protected static void compare(AOffset o, int[] v) {
-		AIterator i = o.getIterator();
-		if(v[0] != i.value())
-			fail("incorrect result using : " + o.getClass().getSimpleName() + " expected: " + Arrays.toString(v)
-				+ " but was :" + o.toString());
-		for(int j = 1; j < v.length; j++) {
-			i.next();
-			if(v[j] != i.value())
-				fail("incorrect result using : " + o.getClass().getSimpleName() + " expected: " + Arrays.toString(v)
-					+ " but was :" + o.toString());
-		}
-		if(i.getOffsetsIndex() != o.getOffsetsLength())
-			fail("The allocated offsets are longer than needed: idx " + i.getOffsetsIndex() + " vs len "
-				+ o.getOffsetsLength() + "\n" + Arrays.toString(v));
+	@Test
+	public void testIteratorToString() {
+		AOffsetIterator a = o.getOffsetIterator();
+		if(a != null)
+			a.toString();
+
+		AIterator b = o.getIterator();
+		if(b != null)
+			b.toString();
 	}
 
-	protected static void compareOffsetIterator(AOffset o, int[] v) {
-		AOffsetIterator i = o.getOffsetIterator();
-		if(v[0] != i.value())
-			fail("incorrect result using : " + o.getClass().getSimpleName() + " expected: " + Arrays.toString(v)
-				+ " but was :" + o.toString());
-		for(int j = 1; j < v.length; j++) {
-			i.next();
-			if(v[j] != i.value())
+	@Test
+	public void testIteratorSkipToOverEnd() {
+		AIterator a = o.getIterator();
+		if(a != null) {
+			a.skipTo(Integer.MAX_VALUE);
+		}
+	}
+
+	@Test
+	public void testNextOn1Element() {
+		if(data.length == 1) {
+			AIterator a = o.getIterator();
+			a.next();
+			a.next();
+			a.next();
+
+			AOffsetIterator b = o.getOffsetIterator();
+			b.next();
+			b.next();
+			// should be possible... but not really relevant.
+		}
+	}
+
+	@Test
+	public void sliceFirst100() {
+		slice(0, 100);
+	}
+
+	@Test
+	public void sliceToLast() {
+		if(data.length > 1)
+			slice(0, data[data.length - 1]);
+	}
+
+	@Test
+	public void sliceToLastMissingFirst100() {
+		if(data.length > 1)
+			slice(100, data[data.length - 1]);
+	}
+
+	@Test
+	public void slice100to10000() {
+		slice(100, 10000);
+	}
+
+	@Test
+	public void slice1to4() {
+		slice(1, 4);
+	}
+
+	@Test
+	public void sliceAllSpecific() {
+		if(data.length > 1)
+			slice(data[0], data[data.length - 1] + 1);
+	}
+
+	private void slice(int l, int u) {
+		try {
+
+			OffsetSliceInfo a = o.slice(l, u);
+			a.offsetSlice.toString();
+			if(data.length > 0 && data[data.length - 1] > u) {
+
+				AIterator it = a.offsetSlice.getIterator();
+				int i = 0;
+				while(i < data.length && data[i] < l)
+					i++;
+
+				while(data[i] < u) {
+					assertEquals(data[i] - l, it.value());
+					if(a.offsetSlice.getOffsetToLast() > it.value())
+						it.next();
+					i++;
+				}
+			}
+		}
+		catch(Exception e) {
+			e.printStackTrace();
+			fail("Failed to slice first 100");
+		}
+
+	}
+
+	@Test
+	public void append() {
+		if(data.length > 0) {
+			final int ll = data[data.length - 1] + 1;
+			final AOffset r = o.append(o, ll);
+			compareAppendOne(r, ll);
+		}
+	}
+
+	@Test
+	public void append_v2() {
+		if(data.length > 0) {
+			final int ll = data[data.length - 1] + 3;
+			final AOffset r = o.append(o, ll);
+			compareAppendOne(r, ll);
+		}
+	}
+
+	@Test
+	public void appendN_one() {
+		if(data.length > 0) {
+			final int ll = data[data.length - 1] + 1;
+			final AOffset r = o.appendN(new AOffsetsGroup[] {new Con(o), new Con(o)}, ll);
+			compareAppendOne(r, ll);
+		}
+	}
+
+	@Test
+	public void appendN_one_v2() {
+		if(data.length > 0) {
+			final int ll = data[data.length - 1] + 3;
+			final AOffset r = o.appendN(new AOffsetsGroup[] {new Con(o), new Con(o)}, ll);
+			compareAppendOne(r, ll);
+		}
+	}
+
+	@Test
+	public void compareAppend() {
+		if(data.length > 0) {
+			final int ll = data[data.length - 1] + 3;
+			final AOffset r1 = o.appendN(new AOffsetsGroup[] {new Con(o), new Con(o)}, ll);
+			final AOffset r2 = o.append(o, ll);
+			compare(r1, r2);
+		}
+	}
+
+	@Test
+	public void compareAppend_v2() {
+		if(data.length > 0) {
+			final int ll = data[data.length - 1] + 100;
+			final AOffset r1 = o.appendN(new AOffsetsGroup[] {new Con(o), new Con(o)}, ll);
+			final AOffset r2 = o.append(o, ll);
+			compare(r1, r2);
+		}
+	}
+
+	@Test
+	public void compareAppend_v3() {
+		if(data.length > 0) {
+			final int ll = data[data.length - 1] + 1000;
+			final AOffset r1 = o.appendN(new AOffsetsGroup[] {new Con(o), new Con(o)}, ll);
+			final AOffset r2 = o.append(o, ll);
+			compare(r1, r2);
+		}
+	}
+
+	@Test
+	public void compareAppend_ot() {
+		if(data.length > 0) {
+			final int ll = data[data.length - 1] + 1000;
+			final AOffset r1 = o.appendN(new AOffsetsGroup[] {new Con(o), new Con(OffsetFactory.createOffset(data))},
+				ll);
+			final AOffset r2 = o.append(o, ll);
+			compare(r1, r2);
+		}
+	}
+
+	@Test
+	public void compareAppend_2x() {
+
+		try {
+
+			if(data.length > 0) {
+
+				final int ll = data[data.length - 1] + 100;
+				final AOffset r = o.appendN(new AOffsetsGroup[] {new Con(o), new Con(o), new Con(o)}, ll);
+				final AOffset t2 = o.append(o, ll).append(o, ll * 2);
+				compare(r, t2);
+			}
+		}
+		catch(Exception e) {
+			e.printStackTrace();
+			fail(e.getMessage());
+		}
+	}
+
+	@Test
+	public void compareAppend_2x_v2() {
+
+		try {
+
+			if(data.length > 0) {
+
+				final int ll = data[data.length - 1] + 280;
+				final AOffset r = o.appendN(new AOffsetsGroup[] {new Con(o), new Con(o), new Con(o)}, ll);
+				final AOffset t2 = o.append(o, ll).append(o, ll * 2);
+				compare(r, t2);
+			}
+		}
+		catch(Exception e) {
+			e.printStackTrace();
+			fail(e.getMessage());
+		}
+	}
+
+	private void compareAppendOne(AOffset r, int ll) {
+		if(r.getSize() != data.length * 2)
+			fail("Invalid return length:   is: " + o.getSize() + " Should be " + (data.length * 2));
+		final AIterator i = r.getIterator();
+		if(o.getSize() > 0) {
+			if(i.value() != data[0])
+				fail("incorrect value");
+			for(int j = 1; j < data.length; j++) {
+				i.next();
+				if(data[j] != i.value())
+					fail("Incorrect value: expected: " + (data[j]) + " Was: " + i.value());
+
+			}
+			for(int j = data.length; j < data.length * 2; j++) {
+				i.next();
+				if(data[j - data.length] + ll != i.value())
+					fail("Incorrect value: expected: " + (data[j - data.length] + ll) + " Was: " + i.value());
+
+			}
+			assertEquals(data[data.length - 1] + ll, i.value());
+			assertEquals(data[data.length - 1] + ll, r.getOffsetToLast());
+		}
+	}
+
+	@Test
+	public void moveIndex1() {
+
+		AOffset b = o.moveIndex(10);
+		compareMoved(b, data, -10);
+	}
+
+	@Test
+	public void getLength(){
+		assertTrue(o.getLength() + 1 >= data.length);
+	}
+
+
+	private void compareMoved(AOffset o, int[] v, int m) {
+		AIterator i = o.getIterator();
+		if(o.getSize() != v.length) {
+			fail("Incorrect result sizes : " + o + " " + Arrays.toString(v));
+		}
+		if(o.getSize() > 0) {
+			assertEquals(o.getOffsetToLast(), v[v.length - 1] + m);
+			if(v[0] + m != i.value())
 				fail("incorrect result using : " + o.getClass().getSimpleName() + " expected: " + Arrays.toString(v)
 					+ " but was :" + o.toString());
+			for(int j = 1; j < v.length; j++) {
+				i.next();
+				if(v[j] + m  != i.value())
+					fail("incorrect result using : " + o.getClass().getSimpleName() + " expected: " + Arrays.toString(v)
+						+ " but was :" + o.toString());
+			}
+		}
+	}
+
+	private class Con implements AOffsetsGroup {
+
+		AOffset a;
+
+		protected Con(AOffset a) {
+			this.a = a;
+		}
+
+		@Override
+		public AOffset getOffsets() {
+			return a;
+		}
+
+	}
+
+	public static void compare(AOffset o, int[] v) {
+		AIterator i = o.getIterator();
+		if(o.getSize() != v.length) {
+			fail("Incorrect result sizes : " + o + " " + Arrays.toString(v));
+		}
+		if(o.getSize() > 0) {
+			assertEquals(o.getOffsetToLast(), v[v.length - 1]);
+			if(v[0] != i.value())
+				fail("incorrect result using : " + o.getClass().getSimpleName() + " expected: " + Arrays.toString(v)
+					+ " but was :" + o.toString());
+			for(int j = 1; j < v.length; j++) {
+				i.next();
+				if(v[j] != i.value())
+					fail("incorrect result using : " + o.getClass().getSimpleName() + " expected: " + Arrays.toString(v)
+						+ " but was :" + o.toString());
+			}
+		}
+	}
+
+	public static void compare(AOffset o, AOffset b) {
+		assertEquals(o.getOffsetToFirst(), b.getOffsetToFirst());
+		assertEquals(o.getOffsetToLast(), b.getOffsetToLast());
+
+		AIterator io = o.getIterator();
+		AIterator ib = b.getIterator();
+
+		assertEquals(io.value(), ib.value());
+		int s = o.getSize();
+		assertEquals(s, b.getSize());
+		for(int i = 0; i < s - 1; i++) {
+			assertEquals(io.value(), ib.value());
+			io.next();
+			ib.next();
+		}
+		assertEquals(io.value(), ib.value());
+		assertEquals(io.value(), b.getOffsetToLast());
+
+	}
+
+	public static void compareOffsetIterator(AOffset o, int[] v) {
+		if(o.getSize() != v.length) {
+			fail("Incorrect result sizes : " + o + " " + Arrays.toString(v));
+		}
+		if(o.getSize() > 0) {
+			AOffsetIterator i = o.getOffsetIterator();
+			if(v[0] != i.value())
+				fail("incorrect result using : " + o.getClass().getSimpleName() + " expected: " + Arrays.toString(v)
+					+ " but was :" + o.toString());
+			for(int j = 1; j < v.length; j++) {
+				i.next();
+				if(v[j] != i.value())
+					fail("incorrect result using : " + o.getClass().getSimpleName() + " expected: " + Arrays.toString(v)
+						+ " but was :" + o.toString());
+			}
 		}
 	}
 }
